using System;
using System.Collections;
using System.Collections.Generic;
using _Combo;
using _Core;
using UnityEngine;
using UnityEngine.UI;

namespace _Combo
{
    [ViewPrefab("Prefabs/ComboAlertView")]
    internal class AlertView : View<AlertView>
    {
        public Text titleTxt;
        public Text contentTxt;
        public Button confirmBtn;
        public Button cancelBtn;
        public Text confirmBtnTxt;
        public Text cancelBtnTxt;
        public RectTransform titlePanel;
        public RectTransform contentPanel;
        private Action OnConfirm;
        private Action OnCancel;

        public void ShowCancelBtn() {
            cancelBtn.gameObject.SetActive(true);
        }
        public void HideCancelBtn() {
            cancelBtn.gameObject.SetActive(false);
        }
        public void HideConfirmBtn() {
            confirmBtn.gameObject.SetActive(false);
        }
        public void SetTitle(string title) {
            titleTxt.text = title;
        }
        public void SetMessage(string content) {
            contentTxt.text = content;
        }
        public void SetConfirmBtnTitle(string title) {
            confirmBtnTxt.text = title;
        }
        public void SetCancelBtnTitle(string title, int time = 0) {
            cancelBtnTxt.text = title;
            if(time <= 0) {
                return;
            }
            cancelBtnTxt.text = title + $"( {time}s )";
            StartCoroutine(StartCountdown(title, time));
        }
 
        IEnumerator StartCountdown(string title, int totalTime)
        {
            int remainingTime = totalTime;
            float startTime = Time.realtimeSinceStartup;
            float countdownDuration = remainingTime;

            while (remainingTime >= 0)
            {
                float elapsedTime = Time.realtimeSinceStartup - startTime;
                remainingTime = Mathf.CeilToInt(countdownDuration - elapsedTime);

                if (remainingTime < 0)
                {
                    OnCancel.Invoke();
                    yield return null;
                }
                else
                {
                    cancelBtnTxt.text = title + $"( {remainingTime}s )";
                    yield return new WaitForSecondsRealtime(1f);
                }
            }
        }

        public void SetConfirmCallback(Action OnConfirm) {
            this.OnConfirm = OnConfirm;
        }
        public void SetCancelCallback(Action OnCancel) {
            this.OnCancel = OnCancel;
        }
        
        void Awake() {
            confirmBtn.onClick.AddListener(OnClickConfirmBtn);
            cancelBtn.onClick.AddListener(OnClickCancelBtn);
        }
        void OnDestroy() {
            confirmBtn.onClick.RemoveListener(OnClickConfirmBtn);
            cancelBtn.onClick.AddListener(OnClickCancelBtn);
        }
        void OnClickConfirmBtn() {
            OnConfirm.Invoke();
        }
        void OnClickCancelBtn() {
            OnCancel.Invoke();
        }
        protected override IEnumerator OnHide()
        {
            yield return null;
        }

        protected override IEnumerator OnShow()
        {
            LayoutRebuilder.ForceRebuildLayoutImmediate(titlePanel);
            LayoutRebuilder.ForceRebuildLayoutImmediate(contentPanel);
            yield return null;
        }
    }
}
