using System.Collections;
using UnityEngine;
using UnityEngine.UI;
using _Core;
using _Combo;
using Seayoo.ComboSDK.Windows.Account.Model.VO;
using System.Collections.Generic;
using Seayoo.ComboSDK.Windows.TapTap;
using System.Linq;
using System.Net;

namespace Seayoo.ComboSDK.Windows.Account.View
{
    public class SeayooAccountPhoneLoginView : MonoBehaviour
    {
        public Button closeBtn;
        public Button enterGameBtn;
        public Button backBtn;
        public Button taptapLoginBtn;
        public Button clearBtn;
        public Button captchaBtn;
        public Button passwordBtn;
        public Toggle privacyToggle;
        public Text enterGameText;
        public HyperlinkText privacyText;
        public Text loginNumberText;
        public Text loginText;
        public Text captchaBtnText;
        public Text passwordBtnText;
        public Text phoneNumberPlaceHolder;
        public Text passwordPlaceHolder;
        public Text otherLoginText;
        public InputField phoneNumber;
        public Color normalColor;
        public Color selectedColor;
        public SeayooAccountCaptchaLoginView seayooAccountCaptchaPanelView;
        public SeayooAccountPasswordLoginView seayooAccountPasswordPanelView;
        public GameObject otherLoginTypePanel;
        public GameObject logoImage;
        public GameObject privacy;
        public GameObject captchaPanel;
        public GameObject passwordPanel;
        public GameObject toggleLoginBtnPanel;
        public Image mask;
        private SeayooAccountPhoneLoginViewType loginType = SeayooAccountPhoneLoginViewType.CAPTCHA;
        private const int MOBILE_LENGTH = 11;

        void Awake()
        {
            EventSystem.Register(this);
            enterGameText.text = I18n.T("login_submit_button");
            phoneNumberPlaceHolder.text = I18n.T("login_or_register_mobile_placeholder");
            loginText.text = I18n.T("login_title");
            captchaBtnText.text = I18n.T("login_otp_tab");
            passwordBtnText.text = I18n.T("login_password_tab");
            passwordPlaceHolder.text = I18n.T("password_placeholder");
            otherLoginText.text = I18n.T("login_other_methods_text");
            gameObject.SetActive(false);
            if(IsContainsOtherModules() && !GlobalParameters.Instance.TaptapDisableTapLogin)
            {
                otherLoginTypePanel.gameObject.SetActive(true);
            }
            if(GlobalParameters.Instance.SeayooAccountHideLogo)
            {
                logoImage.gameObject.SetActive(false); 
                loginText.gameObject.SetActive(true);
            }
            if(GlobalParameters.Instance.SeayooAccountDisablePasswordLogin)
            {
                toggleLoginBtnPanel.gameObject.SetActive(false);
            }
        }

        void Start()
        {
            closeBtn.onClick.AddListener(OnClose);
            enterGameBtn.onClick.AddListener(OnEnterGame);
            backBtn.onClick.AddListener(OnBack);
            captchaBtn.onClick.AddListener(OnOpenCaptchaPanel);
            passwordBtn.onClick.AddListener(OnOpenPasswordPanel);
            clearBtn.onClick.AddListener(OnClearPhoneNum);
            taptapLoginBtn.onClick.AddListener(TapTapLogin);
            phoneNumber.onValueChanged.AddListener(OnPhoneFieldChanged);
        }

        void OnDestroy()
        {
            enterGameBtn.onClick.RemoveListener(OnEnterGame);
            closeBtn.onClick.RemoveListener(OnClose);
            backBtn.onClick.RemoveListener(OnBack);
            captchaBtn.onClick.RemoveListener(OnOpenCaptchaPanel);
            passwordBtn.onClick.RemoveListener(OnOpenPasswordPanel);
            clearBtn.onClick.RemoveListener(OnClearPhoneNum);
            taptapLoginBtn.onClick.RemoveListener(TapTapLogin);
            phoneNumber.onValueChanged.RemoveListener(OnPhoneFieldChanged);
            EventSystem.UnRegister(this);
        }

        [EventSystem.BindEvent]
        void OnGoToView(GoToViewCommand action) {
            if (action.viewType != SeayooAccountViewType.PHONE) 
            {
                Hide();
                return;
            }

            Show();
        }

        [EventSystem.BindEvent]
        public void Refresh(RefreshViewCommand action = null)
        {
            this.enterGameBtn.interactable = true;
        }

        void OnPhoneFieldChanged(string newValue)
        {
            clearBtn.gameObject.SetActive(true);
        }

        public void OnOpenCaptchaPanel()
        {
            loginNumberText.text = I18n.T("login_or_register_mobile_placeholder");
            this.captchaPanel.SetActive(true);
            var captchaText = captchaBtn.GetComponentInChildren<Text>();
            captchaText.color = selectedColor;
            this.passwordPanel.SetActive(false);
            var passwordText = passwordBtn.GetComponentInChildren<Text>();
            passwordText.color = normalColor;
            loginType = SeayooAccountPhoneLoginViewType.CAPTCHA;
        }

        public void OnOpenPasswordPanel()
        {
            loginNumberText.text = I18n.T("mobile_placeholder");
            var captchaText = captchaBtn.GetComponentInChildren<Text>();
            captchaText.color = normalColor;
            this.passwordPanel.SetActive(true);
            var passwordText = passwordBtn.GetComponentInChildren<Text>();
            passwordText.color = selectedColor;
            loginType = SeayooAccountPhoneLoginViewType.PASSWORD;
        }

        public void OnEnterGame()
        {
            if (!seayooAccountCaptchaPanelView.CheckMobile())
            {
                return;
            }
            
            switch(loginType) {
                case SeayooAccountPhoneLoginViewType.CAPTCHA:
                    CapChaLogin();
                    break;
                case SeayooAccountPhoneLoginViewType.PASSWORD:
                    LoginWithPassword();
                    break;
            } 
            
        }

        public void CapChaLogin()
        {
            if (string.IsNullOrEmpty(seayooAccountCaptchaPanelView.vo.otpAction))
            {
                UIController.Instance.ShowToast(I18n.T("otp_not_sent_toast"));
                return;
            }

            if (!seayooAccountCaptchaPanelView.CheckCaptcha())
            {
                return;
            }

            if (!privacyToggle.isOn)
            {
                UIController.Instance.ShowToast(I18n.T("agreements_required_toast"));
                return;
            }
            this.enterGameBtn.interactable = false;
            PhoneLoginCommand.Invoke(new PhoneLoginCommand {
                mobile = phoneNumber.text,
                otp = seayooAccountCaptchaPanelView.captcha.text,
                action = seayooAccountCaptchaPanelView.vo.otpAction,
            });
        }

        public void LoginWithPassword()
        {
            if (!seayooAccountPasswordPanelView.CheckPassword())
            {
                return;
            }
            if (!privacyToggle.isOn)
            {
                UIController.Instance.ShowToast(I18n.T("agreements_required_toast"));
                return;
            }
            CreateCaptchaCommand.Invoke(new CreateCaptchaCommand {
                account = phoneNumber.text,
                action = SeayooAccountPhoneLoginViewTypeCode.ConvertToModule(SeayooAccountPhoneLoginViewType.PASSWORD),
                OnSuccess = resp => {
                    if(resp.captchaId == "")
                    {
                        UIController.Instance.ShowToast(I18n.T("create_captcha_id_null_toast"));
                        return;
                    }
                    if(!string.IsNullOrEmpty(resp.captchaProvider) && !string.IsNullOrEmpty(resp.captchaConfig))
                    {
                        var otherData = new Dictionary<string, string>
                        {
                            { "captcha_id", resp.captchaId },
                            { "captcha_provider", resp.captchaProvider},
                            { "captcha_config", resp.captchaConfig}
                        };
                        string otherDataStr = string.Join("&", otherData.Select(kv => $"{kv.Key}={WebUtility.UrlEncode(kv.Value)}"));
                        string languageStr = string.Join("&", new Dictionary<string, string>{{"lang", I18n.GetLanguageCode()}}.Select(kv => $"{kv.Key}={WebUtility.UrlEncode(kv.Value)}"));
                        string url = $"{GlobalParameters.Instance.SeayooAccountCaptchaUrl}?{otherDataStr}&{languageStr}";
#if UNITY_STANDALONE
                        UIController.Instance.ShowWebView(url, WebViewType.PasswordCreateCaptcha, 100, 100, false, 0.6, true);
#endif
                    }
                    else
                    {
                        enterGameBtn.interactable = false;
                        PasswordLoginCommand.Invoke(new PasswordLoginCommand {
                            username = phoneNumber.text,
                            password = seayooAccountPasswordPanelView.password.text,
                            encrypted = false,
                            captchaId = resp.captchaId
                        });
                    }
                },
                OnFail = () => {
                    UIController.Instance.HideLoading();
                }
            });
        }

        [EventSystem.BindEvent]
        public void CaptchaResult(ReceiveCaptchaResult result)
        {
            if(result.webViewType != WebViewType.PasswordCreateCaptcha)
            {
                return;
            }
            enterGameBtn.interactable = false;
            PasswordLoginCommand.Invoke(new PasswordLoginCommand {
                username = phoneNumber.text,
                password = seayooAccountPasswordPanelView.password.text,
                encrypted = false,
                captchaId = result.captchaId,
                captchaResult = result.captchaResult
            });
        }

        public void OnClose()
        {
            CloseWindowCommand.Invoke(new CloseWindowCommand {
                userCancel = true
            });
        }

        public void Show()
        {
            mask.color = new Color(0f, 0f, 0f, 0.3f);
            Clear();
            if (Config.hideEula)
            {
                privacy.gameObject.SetActive(false);
                privacyToggle.isOn = true;
            }
            backBtn.gameObject.SetActive(UserHistoryManager.Instance.GetUsers().Count != 0);
            Refresh();

            this.gameObject.SetActive(true);
            privacyText.text = I18n.T("login_agreements_default_text", new Dictionary<string, object>()
            {
                {"0", Config.userAgreement},
                {"1", I18n.T("login_user_agreements_text")},
                {"2", Config.privacyPolicy},
                {"3", I18n.T("login_privacy_policy_text")}
            });
            StartCoroutine(SetHyperlinkActionsDelayed()); 
        }

        private IEnumerator SetHyperlinkActionsDelayed()
        {
            // 等待一帧，确保 OnPopulateMesh 执行
            yield return null;

            privacyText.SetHyperlinkClickActions(new Dictionary<string, System.Action<string>>{
                {
                    Config.userAgreement, (link)=>{
#if UNITY_STANDALONE
                        UIController.Instance.ShowWebView(link, WebViewType.General, 100, 100);
#endif
                }},
                {
                    Config.privacyPolicy, (link)=>{
#if UNITY_STANDALONE
                        UIController.Instance.ShowWebView(link, WebViewType.General, 100, 100);
#endif
                }}
            });
        }

        private void Clear()
        {
            seayooAccountCaptchaPanelView.UpdateCountdown(false);
            phoneNumber.text = "";
            seayooAccountCaptchaPanelView.captcha.text = "";
            seayooAccountPasswordPanelView.password.text = "";
            this.privacyToggle.isOn = false;
            OnOpenCaptchaPanel();
        }

        public void Hide()
        {
            this.gameObject.SetActive(false);
        }

        public void OnBack()
        {
            GoToViewCommand.Invoke(new GoToViewCommand{
                viewType = SeayooAccountViewType.TOKEN
            });
        }

        public void OnClearPhoneNum()
        {
            phoneNumber.text = "";
            clearBtn.gameObject.SetActive(false);
        }

        public bool CheckMobile()
        {
            if (string.IsNullOrEmpty(phoneNumber.text))
            {
                UIController.Instance.ShowToast(I18n.T("mobile_required_toast"));
                return false;
            }
            if (phoneNumber.text.Length != MOBILE_LENGTH)
            {
                UIController.Instance.ShowToast(I18n.T("mobile_invalid_toast"));
                return false;
            }
            return true;
        }

        public void TapTapLogin()
        {
            if (!privacyToggle.isOn)
            {
                UIController.Instance.ShowToast(I18n.T("agreements_required_toast"));
                return;
            }
            TapTap.TapTap.Instance.Login();
            Hide();
        }

        private bool IsContainsOtherModules()
        {
            if(Configuration.Instance.Modules.Contains("taptap"))
            {
                return true;
            }
            return false;
        }
    }
}
