using _Combo.Events;
using _Core;
using UnityEngine;
using UnityEngine.UI;

namespace Seayoo.ComboSDK.Windows.Store.View
{
    public class SandboxView : MonoBehaviour
    {
        public Button cancelBtn;
        public Button confirmBtn;
        public Text titleText;
        public Text tipText;
        public Text confirmText;
        public Text productNameTxt;
        public Text productValueTxt;
        private string orderToken;
        void Awake()
        {
            EventSystem.Register(this);
            this.gameObject.SetActive(false);
        }

        void Start()
        {
            titleText.text = I18n.T("sandbox_payment_title");
            tipText.text = I18n.T("sandbox_payment_text");
            confirmText.text = I18n.T("confirm_payment_button");
            cancelBtn.onClick.AddListener(OnClose);
            confirmBtn.onClick.AddListener(OnConfirm);
        }

        void OnDestroy()
        {
            EventSystem.UnRegister(this);
            cancelBtn.onClick.RemoveListener(OnClose);
            confirmBtn.onClick.RemoveListener(OnConfirm);
        }
        public void OnClose()
        {
            PurchaseCancelEvent.Invoke();
            ClosePayViewCommand.Invoke();
        }

        public void OnConfirm()
        {
            SeayooStoreManager.Instance.RequestPayment(orderToken, SeayooStoreManager.Instance.GetPaymentMethod(PaymentMethods.SEAYOO_SANDBOX));
        }

        [EventSystem.BindEvent]
        public void ShowPayView(GoToViewCommand action)
        {
            if(action.type != PayViewType.PAY_SANDBOX)
            {
                this.gameObject.SetActive(false);
                return;
            }
            this.gameObject.SetActive(true);
            productNameTxt.text = action.productName;
            productValueTxt.text = action.productValue;
        }

        [EventSystem.BindEvent]
        public void ShowTextInfo(ProductInfoCommand action)
        {
            orderToken = action.orderToken;
        }
    }
}